import chimera
from chimera.baseDialog import ModelessDialog

class MapSumsDialog(ModelessDialog):
	name = "Map Sums"
	help = "ContributedSoftware/MapSums/mapsums.html"
	buttons = ("OK", "Close")
	default = "OK"

	def __init__(self, *args, **kw):
		# Create dialog
		ModelessDialog.__init__(self, *args, **kw)
		# Register dialog to make it appear at the bottom
		# of Tools menu
		from chimera.extension import manager
		manager.registerInstance(self)
		# Handle session saving and closing
		import chimera
		self._closeSesHandler = chimera.triggers.addHandler(
						chimera.CLOSE_SESSION,
						self.emQuit, None)

	def destroy(self):
		# Delete handlers
		if self._closeSesHandler:
			chimera.triggers.deleteHandler(chimera.CLOSE_SESSION,
							self._closeSesHandler)
			self._closeSesHandler = None
		# Deregister from extension manager
		from chimera.extension import manager
		manager.deregisterInstance(self)
		# Destroy dialog
		ModelessDialog.destroy(self)

	def fillInUI(self, parent):
		import Tkinter
		parent.columnconfigure(2, weight=1)

		# Volume selector
		l = Tkinter.Label(parent, text="Volume:")
		from VolumeViewer.volume import Volume
		def isVolume(m):
			return isinstance(m, Volume)
		l.grid(row=0, column=0, sticky="e")
		from chimera.widgets import ModelOptionMenu
		self.volumeOption = ModelOptionMenu(parent, filtFunc=isVolume)
		self.volumeOption.grid(row=0, column=1,
						columnspan=2, sticky="ew")

		# Atoms selector
		l = Tkinter.Label(parent, text="Atoms:")
		l.grid(row=1, rowspan=2, column=0, sticky="e")
		self.atomsVar = Tkinter.IntVar(parent)
		b = Tkinter.Radiobutton(parent, text="selected",
						command=self._atomsChanged,
						variable=self.atomsVar, value=0)
		b.grid(row=1, column=1, columnspan=2, sticky="w")
		b = Tkinter.Radiobutton(parent, text="specifier",
						command=self._atomsChanged,
						variable=self.atomsVar, value=1)
		b.grid(row=2, column=1, sticky="w")
		self.atomsVar.set(0)
		self.specVar = Tkinter.StringVar(parent)
		self.specEntry = Tkinter.Entry(parent,
						textvariable=self.specVar)
		self.specEntry.grid(row=2, column=2, sticky="ew")
		self.specVar.set("@CA,CB")

		# Radius selector
		l = Tkinter.Label(parent, text="Radii:")
		l.grid(row=3, column=0, sticky="e")
		self.radiiVar = Tkinter.StringVar(parent)
		self.radiiEntry = Tkinter.Entry(parent,
						textvariable=self.radiiVar)
		self.radiiEntry.grid(row=3, column=1, columnspan=2, sticky="ew")
		self.radiiVar.set("3,5")

		# grid spacing selector
		l = Tkinter.Label(parent, text="Grid spacing:")
		l.grid(row=4, column=0, sticky="e")
		self.spacingVar = Tkinter.StringVar(parent)
		self.spacingEntry = Tkinter.Entry(parent,
						textvariable=self.spacingVar)
		self.spacingEntry.grid(row=4, column=1, columnspan=2,
								sticky="ew")
		self.spacingVar.set("0.3")

	def _atomsChanged(self):
		if self.atomsVar.get() == 1:
			state = "normal"
		else:
			state = "disabled"
		self.specEntry.config(state=state)

	def Apply(self):
		m = self.volumeOption.getvalue()
		if m is None:
			self.enter()
			from chimera import UserError
			raise UserError("no volume selected")
		maps = [ m ]

		atomsMode = self.atomsVar.get()
		from chimera import selection
		if atomsMode == 1:
			# use specifier
			spec = self.specVar.get()
			if not spec:
				spec = "#"
			sel = selection.OSLSelection(spec)
			atoms = sel.atoms()
			if not atoms:
				self.enter()
				from chimera import UserError
				raise UserError("no atoms match specification")
		else:
			# use selection
			atoms = selection.currentAtoms()
			if not atoms:
				self.enter()
				from chimera import UserError
				raise UserError("no atoms selected")

		radiiStr = self.radiiVar.get()
		try:
			r = [ float(v) for v in radiiStr.split(',') ]
			if len(r) == 0:
				raise ValueError("not a list of radii")
		except ValueError:
			self.enter()
			from chimera import UserError
			raise UserError("radii must be a comma-separated "
					"list of floating point numbers")

		try:
			spacing = float(self.spacingVar.get())
		except ValueError:
			self.enter()
			from chimera import UserError
			raise UserError("spacing must be a "
					"floating point number")

		from cmdline import mapsums
		mapsums(maps, atoms, radiiStr, spacing)

	#
	# Implement API called by extension manager from items in Tools menu
	#
	def emRaise(self):
		self.enter()

	def emHide(self):
		self.Close()

	def emQuit(self, *args):
		self.destroy()

from chimera import dialogs
dialogs.register(MapSumsDialog.name, MapSumsDialog)


class MapSumsResultsDialog(ModelessDialog):
	buttons = ("Save", "Close")
	default = "Close"

	def __init__(self, sums, volume, atoms, radii, gridSize, gridSpacing,
								*args, **kw):
		import weakref
		self.sums = sums
		self.volumeName = volume.name
		self.volume = weakref.ref(volume)
		self.numAtoms = len(atoms)
		self.atomRowMap = weakref.WeakKeyDictionary()
		self.rowAtomMap = weakref.WeakValueDictionary()
		for i, a in enumerate(atoms):
			self.atomRowMap[a] = i
			self.rowAtomMap[i] = a
		self.radii = radii
		self.gridSize = gridSize
		self.gridSpacing = gridSpacing
		# Create dialog
		self.title = "Map sums for %s" % volume.name
		ModelessDialog.__init__(self, *args, **kw)
		# Register dialog to make it appear at the bottom
		# of Tools menu
		from chimera.extension import manager
		manager.registerInstance(self)
		# Handle session saving and closing
		import chimera
		self._closeSesHandler = chimera.triggers.addHandler(
						chimera.CLOSE_SESSION,
						self.destroy, None)
		self._selHandler = chimera.triggers.addHandler(
						"selection changed",
						self._selectionChanged, None)

	def destroy(self, *args):
		# Delete handlers
		if self._closeSesHandler:
			chimera.triggers.deleteHandler(chimera.CLOSE_SESSION,
							self._closeSesHandler)
			self._closeSesHandler = None
		# Deregister from extension manager
		from chimera.extension import manager
		manager.deregisterInstance(self)
		# Destroy dialog
		ModelessDialog.destroy(self)

	def fillInUI(self, parent):
		import Tkinter
		lines = list()
		t = list()
		t.append("Average density near %d atoms." % self.numAtoms)
		t.append("Sampling grid spacing %.4g." % self.gridSpacing)
		lines.append("  ".join(t))
		t = list()
		for r, size in zip(self.radii, self.gridSize):
			t.append("%d points at radius %.1g." % (size, r))
		lines.append("  ".join(t))
		l = Tkinter.Label(parent, text='\n'.join(lines),
					anchor="w", justify="left")
		l.pack(side="top", fill="x")

		from CGLtk import Table
		table = Table.SortableTable(parent)
		table.pack(side="top", fill="both", expand=True)
		table.addColumn("Chain", self._getChain)
		table.addColumn("Residue", self._getResidue)
		table.addColumn("Atom", self._getAtom)
		for i, r in enumerate(self.radii):
			def _getSum(inst, n=i, s=self):
				return s._getSum(inst, n)
			table.addColumn("%.1g" % r, _getSum, format="%8.3f")
		table.setData(range(self.numAtoms))
		table.launch(browseCmd=self._rowChanged)
		self.table = table

	def _getChain(self, row):
		try:
			return self.rowAtomMap[row].residue.id.chainId
		except KeyError:
			return '-'

	def _getResidue(self, row):
		try:
			return self.rowAtomMap[row].residue.id.position
		except KeyError:
			return -1

	def _getAtom(self, row):
		try:
			return self.rowAtomMap[row].name
		except KeyError:
			return "-"

	def _getSum(self, row, col):
		return self.sums[col][row]

	def Save(self):
		from OpenSave import SaveModal
		files = SaveModal(title="Save Map Sums").run(self.uiMaster())
		if not files:
			return
		filename = files[0][0]	# files should be a 1-list of 2-tuple
					# of (filename, filetype)
		atoms = [ self.rowAtomMap.get(i, None)
				for i in range(self.numAtoms) ]
		from cmdline import printMapResults
		with file(filename, "w") as f:
			printMapResults(f, self.sums, self.volumeName,
						atoms, self.radii,
						self.gridSize, self.gridSpacing)

	def _rowChanged(self, selected):
		atoms = list()
		for row in selected:
			try:
				atoms.append(self.rowAtomMap[row])
			except KeyError:
				pass
		from chimera import selection
		sel = selection.ItemizedSelection(atoms)
		selection.setCurrent(sel)

	def _selectionChanged(self, triggerName, ignore, selected):
		rows = list()
		for atom in selected.atoms():
			try:
				rows.append(self.atomRowMap[atom])
			except KeyError:
				pass
		self.table.select(rows)
		if rows:
			self.table.showRow(rows[0])
